/*!
  \file User.cpp

  \brief User-related stuff, definations

  $Id: User.cpp,v 1.10 2006/05/31 12:09:32 tarle Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   19.08.2005 13:48:29 MSD
*/

#include "h/User.h"

using namespace Ice;

UserAttrsDesc_t::AttrsDescription()
{
  Add(uHandle,               "HANDLE");
  Add(uFullName,             "FULLNAME");             
  Add(uBirthday,             "BIRTHDAY");
  Add(uSex,                  "SEX");
  Add(uLanguage,             "LANGUAGE");
  Add(uCountry,              "COUNTRY");
  Add(uProvince,             "PROVINCE");
  Add(uCity,                 "CITY");
  Add(uPhoneHome,            "PHONE_HOME");
  Add(uPhoneOffice,          "PHONE_OFFICE");
  Add(uPhoneMobile,          "PHONE_MOBILE");
  Add(uHomePage,             "HOMEPAGE");
  Add(uAbout,                "ABOUT");
  Add(uHasCallEquipment,     "HASCALLEQUIPMENT");
  Add(uIsVideoCapable,       "IS_VIDEO_CAPABLE");
  Add(uBuddyStatus,          "BUDDYSTATUS", 1, 1);
  Add(uIsAuthorized,         "ISAUTHORIZED", 1, 1);
  Add(uIsBlocked,            "ISBLOCKED", 1, 1);
  Add(uDisplayName,          "DISPLAYNAME");
  Add(uOnlineStatus,         "ONLINESTATUS");
  Add(uLastOnlineTimeStamp,  "LASTONLINETIMESTAMP");
  Add(uCanLeaveVM,           "CAN_LEAVE_VM");
  Add(uSpeeddial,            "SPEEDDIAL", 1, 1);
  Add(uReceiveAuthRequest,   "RECEIVEDAUTHREQUEST");
  Add(uMoodText,             "MOOD_TEXT");
  Add(uAliases,              "ALIASES");
  Add(uTimezone,             "TIMEZONE");
  Add(uIsCFActive,           "IS_CF_ACTIVE");
};

User::User(SkypeConnection& _sc, const std::string& _handle, const LogFunc_t& _log_func) 
     : 	Parent_t(_sc, *this, "USER", _handle, _log_func)
{
  Init(); 	
};	

User::User(SkypeConnection& _sc, const std::string& _handle,  const ObjErrHandler_t& _eh, const LogFunc_t& _log_func)
    : Parent_t(_sc, *this, "USER", _handle, _eh, _log_func)
{
  Init();	
};

User::LiveMode_t User::IsAlive() const
{
/*
  const_iterator p = find(cmStatus);
  if(p == end()) return unknown;

  const char* live[2] = { "SENDING", "RECEIVED" };
  for(int i=0;i<2;i++) if(p->second == live[i]) return alive;
  */
  return finished;
};

void User::Init()
{
  struct Parser_t::SkypeObjRegEx re;
  re.resp = boost::regex("^\\s*USER\\s+([^\\s]+)\\s+(\\w+)\\s+(.*)$");
  re.resp_value_pos = 3;
  re.event_attr_pos = 2;
  re.event_value_pos = 3;
  parser.SetRegEx(re);
};

UserSearchDesc_t::AttrsDescription()
{
  Add(srFriends,			"FRIENDS");
  Add(srUsers,				"USERS");
  Add(srUsersWaitingMyAuthorization,	"USERSWAITINGMYAUTHORIZATION");
};


UserManager::UserManager(SkypeConnection& _sc, const LogFunc_t& _log_func) 
   : LogAbility(_log_func), Parent_t(_sc, *this, _log_func), sc(_sc)
{ Init(); };

UserManager::UserManager(SkypeConnection& _sc,  const User::ObjErrHandler_t& _eh, const LogFunc_t& _log_func)
	    : LogAbility(_log_func), Parent_t(_sc, *this, _eh, _log_func), sc(_sc)
{ Init(); };

void UserManager::Init()
{	
  AddEvent("^\\s*USER\\s+([^\\s]+)\\s+(\\w+)\\s+(.*)$");
};	

std::string UserManager::ParseEvent(const boost::cmatch& _match)
{
  Assert(_match.size() == 4);
 
//  log(LlError, "%s [%s %s]", __FUNCTION__, _match[1].str().c_str(), _match[2].str().c_str());
  
  return (_match[1].str());	
};	


