#include "h/Privileges.h"
#include "h/SkypeConnection.h"

#include "SkypeTest.h"

typedef StreamLogger<LvlTimeLogPrefix> MyLogger_t;
MyLogger_t logger(std::cout);
LogFunc_t lf(&logger, &MyLogger_t::Do);
using namespace Ice;

void DefaultCallback(const std::string& s)
{
	fprintf(stdout, "%s: '%s'\n", __FUNCTION__, s.c_str());	
};


template < class SO_t >
void printObjAttrs(SO_t& so)
{
	std::cout << __FUNCTION__ << std::endl;
//--- Tutorial example code ---
	typedef typename SO_t::const_iterator CI;
	const typename SO_t::AttrsDescription_t desc;
	
	for(CI p = so.begin(); p != so.end(); ++p) 
	{ 
		std::cout << "Obj['" << desc[p->first] << "'] = '";
		std::cout << p->second << "'" << std::endl;
	};
//--- End of tutorial example code ---
}

int main(int argc, char* argv[])
{
    SkypeConnection sk(argv[0], SimpleHandler_t(DefaultCallback), lf);

    sk.Attach();
    sk << DisableIds << "PROTOCOL 99" << flush << EnableIds;

	Profile profile(sk, sk.GetLog());

	std::list<ProfileAttrs_t> attrs;
	attrs.push_back(prPSTNBalance);
	attrs.push_back(prPSTNBalanceCurr);
	attrs.push_back(prFullName);
	attrs.push_back(prBirthday);
	attrs.push_back(prSex);
	attrs.push_back(prLanguages);
	attrs.push_back(prCountry);
	attrs.push_back(prProvince);
	attrs.push_back(prCity);
	attrs.push_back(prPhoneHome);
	attrs.push_back(prPhoneOffice);
	attrs.push_back(prPhoneMobile);
	attrs.push_back(prHomePage);
	attrs.push_back(prAbout);
	attrs.push_back(prMoodText);
	attrs.push_back(prTimeZone);
	attrs.push_back(prCallNoAnswerTimeout);
	attrs.push_back(prCallForwardRules);  

	std::list<ProfileAttrs_t>::const_iterator p = attrs.begin(); 
	for(;p != attrs.end(); ++p) {
		profile.AskAttr(*p);
	};

	const boost::posix_time::seconds tm(1);
	ProcessSkypeMsgs(sk, tm);
	ProcessSkypeMsgs(sk, tm);

	printObjAttrs<Profile>(profile);

	return 0;
}


