/*!
  \file SkypeObj.cpp

  \brief Several tests of templates of the unit SkypeObj 

  $Id: SkypeObj.cpp,v 1.3 2005/09/26 10:21:01 vit Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   13.08.2005 13:53:16 MSD
*/
#include "SkypeTest.h"

#include "h/SkypeObj.h"

using namespace boost::unit_test;

typedef enum { tlFirst, tlSecond, tlThird, tlLast } TestAttrs_t;	  

namespace Ice {
	
typedef AttrsDescription<TestAttrs_t>  TestAttrsDesc_t;

template <>
class AttrsDescription<TestAttrs_t> : public AttrsDescriptionBase<TestAttrs_t> {
public:	
	AttrsDescription(); 	
};

}; // namespace Ice

using namespace Ice;

TestAttrsDesc_t::AttrsDescription()
{
  Add(tlFirst, "First");
  Add(tlSecond, "Secondsdsdfdf");
  Add(tlThird, "Third");
  Add(tlSecond, "Second", 1, 1, 1);
};	

void AttrsDescriptionTest()
{
  TestAttrsDesc_t tst;
  BOOST_CHECK_EQUAL(tst["Second"], tlSecond);
  BOOST_CHECK_EQUAL(tst[tlLast].length(), 0);
  BOOST_CHECK(tst[tlSecond] == "Second");
  BOOST_CHECK_EQUAL(tst["First"], tlFirst);
 
  
  BOOST_CHECK_EQUAL(tst.GetDetails(tlSecond).can_get, 1);
  BOOST_CHECK_EQUAL(tst.GetDetails(tlSecond).can_set, 1);
  BOOST_CHECK_EQUAL(tst.GetDetails(tlSecond).reported_at_obj_creation, 1);

  // Default arguments check
  BOOST_CHECK_EQUAL(tst.GetDetails(tlFirst).can_get, 1);
  BOOST_CHECK_EQUAL(tst.GetDetails(tlFirst).can_set, 0);
  BOOST_CHECK_EQUAL(tst.GetDetails(tlFirst).reported_at_obj_creation, 0);

  // Exceptions checking
  BOOST_CHECK_THROW(tst.GetDetails(tlLast), SkypeInvalidAttr);
  BOOST_CHECK_THROW(tst["Last"], SkypeInvalidAttr);

  try {
	tst.GetDetails(tlLast);	  
  } catch(SkypeInvalidAttr& e) {
	BOOST_CHECK_EQUAL(e.GetAttr(), tlLast);	  
  };	 

  try {
	tst[tlLast];	  
  } catch(SkypeInvalidAttr& e) {
	BOOST_CHECK_EQUAL(e.GetAttr(), -1);	  
  };	 

  // Iterating through description
  typedef TestAttrsDesc_t::const_iterator CI;
  std::vector<TestAttrs_t> tst_vec, etalon;  
  for(CI p = tst.begin();p != tst.end(); ++p) {
	tst_vec.push_back(p->first);	  
  };	  
  etalon.push_back(tlFirst);
  etalon.push_back(tlSecond);
  etalon.push_back(tlThird);

  CmpVecs(tst_vec, etalon);
};

test_suite* init_unit_test_suite(int argc, char* argv[])
{
    test_suite* test= BOOST_TEST_SUITE("SkypeObj and related test");
    
    test->add(BOOST_TEST_CASE(&AttrsDescriptionTest));
    return test;
};

