/*!
  \file User.cpp

  \brief  Tests of user object & user manager

  $Id: User.cpp,v 1.10 2006/06/23 13:21:53 vit Exp $

  \author (c) Ice Brains Software, llc.  http://www.icebrains-soft.com
  \date   12.08.2005 00:08:49 MSD
*/

#include <boost/test/unit_test.hpp>
#include <boost/test/unit_test_suite.hpp>

#include "h/User.h"

#include "SkypeTest.h"

using namespace boost::unit_test;
using namespace Ice;

SkypeConnection* SC;

void DefaultCallback(const std::string& s)
{
	fprintf(stderr, "%s: '%s'\n", __FUNCTION__, s.c_str());	
};

int ObjCreatedCntr = 0;
std::vector<UserAttrs_t> getted_attrs;

UserManager* UM;

void UserChanged(User& user, const UserAttrs_t& attr, const std::string& value);

bool ObjCreatedSlot(User& user) 
{
	fprintf(stderr, "Create user\n");	
	ObjCreatedCntr++;
	user.GetWatchers().RegisterWatcher(UserWatchers_t::AttrSet_t(), UserChanged, true);
	return true;	
};

void UserChanged(User& user, const UserAttrs_t& attr, const std::string& value)
{
	UserAttrsDesc_t attrs_desc;
	fprintf(stderr, "111111\n");
	fprintf(stderr, "New value of User %s attribute '%s' is '%s'\n", user.GetId().c_str(), attrs_desc[attr].c_str(), value.c_str());	
};

bool UserSearchResult_called = false;
std::vector<User*> search_res;
bool UserSearchResult(const std::vector<User*> _res)
{
	UserSearchResult_called = true;
	BOOST_CHECK(_res.size() > 0);
	fprintf(stderr, "	- Found %d users\n", _res.size());
	search_res = _res;
  
	return false;
};


/*! Search users by 'echo1' string.
	Check follow condition:
		- #UserSearchResult was launched;
		- search results is not blank;
*/
void UserSearchEcho1_Test()
{
	SkypeConnection& sc = *SC;
  
	UserManager um(sc, sc.GetLog());
	UM = &um;
  
	UserWatchers_t::AttrSet_t attrs;
	attrs.insert(uLastOnlineTimeStamp);
	attrs.insert(uOnlineStatus);
	attrs.insert(uHandle);
	attrs.insert(uFullName);
	  
	const boost::posix_time::seconds tm(1);

	fprintf(stderr, "->Start of search user test (search with 'echo1' string) \n");	
	UserSearchResult_called = false;
	um.Search(srUsers, UserSearchResult, attrs, "echo1");
	ProcessSkypeMsgs(sc, tm);

	for(int i = 0; i < 60; ++i) {
		ProcessSkypeMsgs(sc, tm);
	}

	BOOST_CHECK(UserSearchResult_called == true);
	fprintf(stderr, "<-End of search user test\n");	

	um.Squeeze();
};

/*! Search users by 'echo1' string.
	Check follow condition:
		- #UserSearchResult was launched;
		- search result is single (in the meantime 
		IceBrains-Software is single result of that search query);
*/
void UserSearch_icebrains_soft_Test()
{
	SkypeConnection& sc = *SC;
  
	UserManager um(sc, sc.GetLog());
	UM = &um;
  
	UserWatchers_t::AttrSet_t attrs;
	attrs.insert(uLastOnlineTimeStamp);
	attrs.insert(uOnlineStatus);
	attrs.insert(uHandle);
	attrs.insert(uFullName);
	  
	const boost::posix_time::seconds tm(1);

	fprintf(stderr, "->Start of search user test (search with 'icebrains-soft' string) \n");
	UserSearchResult_called = false;
	um.Search(srUsers, UserSearchResult, attrs, "icebrains-soft");
	ProcessSkypeMsgs(sc, tm);

	for(int i = 0; i < 60; ++i) {
		ProcessSkypeMsgs(sc, tm);
	}

	BOOST_CHECK(UserSearchResult_called == true);
	BOOST_CHECK(search_res.size() == 1);
	fprintf(stderr, "<-End of search user test\n");	

	um.Squeeze();
};


test_suite* init_unit_test_suite(int argc, char* argv[])
{
    test_suite* test= BOOST_TEST_SUITE("User and UserManager unit test");
	
//	static UserWatchers_t::AttrSet_t attrs;
//	attrs.insert(uLastOnlineTimeStamp);
//	attrs.insert(uOnlineStatus);
    
 
	static FileLogger<> fl(stderr, LlDbg);
    static SimpleHandler_t def(DefaultCallback);
    static SkypeConnection scl(argv[0], def, LogFunc_t(&fl, &FileLogger<>::operator()));
    scl.Attach();
    scl << DisableIds << "PROTOCOL 99" << flush << EnableIds;

    SC = &scl;
      
    test->add(BOOST_TEST_CASE(&UserSearchEcho1_Test));
    test->add(BOOST_TEST_CASE(&UserSearch_icebrains_soft_Test));
    
    return test;
};

